using System;
using System.Collections;
using UnityEngine;
using UnityEngine.InputSystem;

namespace UtilEssentials.UIDocumentExtenderer
{
    [DefaultExecutionOrder(-10)]
    public class PlayerInputExtender : MonoBehaviour
    {
        static PlayerInputExtender _instance;
        static public PlayerInputExtender instance => _instance;

        public Action SwitchedToKeyboardAndMouse;
        public Action SwitchedToGamepad;
        public Action<object, string> InputsRebound;

        string _controlScheme;
        public string controlScheme => _controlScheme;

        string _controlPath;
        public string controlPath => _controlPath;

        [SerializeField] PlayerInput _playerInput;

        [SerializeField] string _playerControlScheme;
        public string playerControlScheme => _playerControlScheme;

        [SerializeField] string _UIControlScheme;
        public string UIControlScheme => _UIControlScheme;


        public PlayerInput playerInput
        { 
            get
            {
                if (_playerInput == null)
                {
                    _playerInput = new GameObject().AddComponent<PlayerInput>();
                }
                return _playerInput;
            }
        }
            

        void Awake()
        {
            if (_instance == null)
            {
                _instance = this;
            }
            else
            {
                Debug.LogError("Duplicate Player Input Extender");
                Destroy(this);
                return;
            }

            if (playerInput == null)
            {
                _playerInput = FindObjectOfType<PlayerInput>();
            }

        }

        void OnEnable()
        {
            InputSystem.onActionChange += CheckControlSchemeName;
            StartCoroutine(DelayControlSchemeChange());
            IEnumerator DelayControlSchemeChange()
            {
                yield return new WaitForEndOfFrame();
                CheckControlSchemeName(null, new InputActionChange());
            }
        }

        void OnDisable()
        {
            InputSystem.onActionChange -= CheckControlSchemeName;
        }

        void CheckControlSchemeName(object obj, InputActionChange change)
        {
            Action CheckForNewControlPath = () =>
            {
                if (_controlScheme == _playerInput.currentControlScheme)
                {
                    return;
                }
                _controlScheme = _playerInput.currentControlScheme;

                foreach (var scheme in playerInput.actions.controlSchemes)
                {
                    if (!scheme.name.Equals(_controlScheme))
                    {
                        continue;
                    }

                    foreach (var requirement in scheme.deviceRequirements)
                    {
                        bool validPath = false;
                        switch (requirement.controlPath)
                        {
                            case "<Keyboard>":
                                SwitchedToKeyboardAndMouse?.Invoke();
                                validPath = true;
                                break;
                            case "<Gamepad>":
                                SwitchedToGamepad?.Invoke();
                                validPath = true;
                                break;
                            default:
                                continue;
                        }

                        if (validPath)
                        {
                            _controlPath = requirement.controlPath;
                            return;
                        }

                    }
                    Debug.LogWarning($"No control path for control scheme {scheme.name} in {playerInput.actions} setup", this);
                    return;
                }
            };

            CheckForNewControlPath();

            if (change == InputActionChange.BoundControlsChanged)
            {
                OnInputsRebound(obj, _controlPath);
            }
            return;
        }

        void OnInputsRebound(object obj, string currentControlPath)
        {
            InputsRebound?.Invoke(obj, currentControlPath);
        }

        public void SwitchToPlayerControlMap()
        {
            playerInput.SwitchCurrentActionMap(_UIControlScheme);
            playerInput.SwitchCurrentActionMap(_playerControlScheme);
        }

        public void SwitchToUIMap()
        {
            playerInput.SwitchCurrentActionMap(_playerControlScheme);
            playerInput.SwitchCurrentActionMap(_UIControlScheme);
        }
    }
}