using System;
using System.Collections;
using UnityEngine;
using UnityEngine.UIElements;

namespace UtilEssentials.UIDocumentExtenderer
{
    [RequireComponent(typeof(UIDocument)), DefaultExecutionOrder(1)]
    public class UIDocumentExtender : MonoBehaviour
    {
        UIDocument _document;

        VisualElement _currentlyFocused;
        public VisualElement currentlyFocused => _currentlyFocused;

        VisualElement _previouslyFocused;
        public VisualElement previouslyFocused => _previouslyFocused;

        public event Action _UILoaded;

        bool _isLoaded;
        public bool isLoaded => _isLoaded;
        void Awake()
        {
            _document = GetComponent<UIDocument>();
        }

        void OnEnable()
        {
            _isLoaded = false;
            if (UILoader.instance.isLoading)
            {
                UILoader.instance.LoadComplete += OnUILoaded;
                return;
            }
            OnUILoaded();

            PlayerInputExtender.instance.SwitchedToKeyboardAndMouse += SwitchedToMouseAndKeyboard;
            PlayerInputExtender.instance.SwitchedToGamepad += SwitchedToGamepad;
        }

        void OnUILoaded()
        {
            UILoader.instance.LoadComplete -= OnUILoaded;
            _isLoaded = true;

            _document.rootVisualElement.RegisterCallback<FocusInEvent>(evt =>
            {
                _currentlyFocused = evt.target as VisualElement;
            });

            _document.rootVisualElement.RegisterCallback<FocusOutEvent>(evt =>
            {
                var element = evt.target as VisualElement;
                _previouslyFocused = element;
                StartCoroutine(DelayCheck());
                IEnumerator DelayCheck()
                {
                    yield return null;
                    if (_currentlyFocused == element)
                    {
                        _previouslyFocused = null;
                        element.Focus();
                    }
                }
            });

            _UILoaded?.Invoke();

            switch (PlayerInputExtender.instance.controlPath)
            {
                case "<Keyboard>":
                    SwitchedToMouseAndKeyboard();
                    break;
                case "<Gamepad>":
                    SwitchedToGamepad();
                    break;
            }
        }

        private void OnDisable()
        {
            UILoader.instance.LoadComplete -= OnUILoaded;

            PlayerInputExtender.instance.SwitchedToKeyboardAndMouse -= SwitchedToMouseAndKeyboard;
            PlayerInputExtender.instance.SwitchedToGamepad -= SwitchedToGamepad;

            _isLoaded = false;
            _currentlyFocused = null;
            _previouslyFocused = null;
        }

        bool IterateElements(VisualElement element, Func<VisualElement, bool> iterateAction)
        {
            for (int i = 0; i < element.childCount; i++)
            {
                var child = element.ElementAt(i);
                if (iterateAction(child))
                {
                    return true;
                }
                if (IterateElements(child, iterateAction))
                {
                    return true;
                }
            }
            return false;
        }

        void SwitchedToGamepad()
        {
            UnityEngine.Cursor.lockState = CursorLockMode.Locked;
            UnityEngine.Cursor.visible = false;

            if (!_isLoaded)
            {
                return;
            }
            if (_currentlyFocused != null)
            {
                return;
            }
            if (_previouslyFocused != null && _previouslyFocused.focusable)
            {
                _previouslyFocused.Focus();
                return;
            }

            IterateElements(_document.rootVisualElement, (VisualElement element) =>
            {
                if (element.canGrabFocus)
                {
                    element.Focus();
                    return true;
                }
                return false;
            });
        }

        void SwitchedToMouseAndKeyboard()
        {
            UnityEngine.Cursor.lockState = CursorLockMode.Confined;
            UnityEngine.Cursor.visible = true;
        }
    }

}
